<?php
/*
Plugin Name: Highlight Search Terms
Plugin URI: http://w-shadow.com/blog/2009/04/13/highlight-search-terms-plugin/
Description: Highlights search terms within your search results using jQuery. Based on a <a href="http://weblogtoolscollection.com/archives/2009/04/10/how-to-highlight-search-terms-with-jquery/">theme hack by Thaya Kareeson</a>.
Version: 1.1
Author: Janis Elsts
Author URI: http://w-shadow.com/blog/
*/

//Make sure the needed constants are defined
if ( ! defined( 'WP_CONTENT_URL' ) )
	define( 'WP_CONTENT_URL', get_option( 'siteurl' ) . '/wp-content' );
if ( ! defined( 'WP_CONTENT_DIR' ) )
	define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );
if ( ! defined( 'WP_PLUGIN_URL' ) )
	define( 'WP_PLUGIN_URL', WP_CONTENT_URL. '/plugins' );
if ( ! defined( 'WP_PLUGIN_DIR' ) )
	define( 'WP_PLUGIN_DIR', WP_CONTENT_DIR . '/plugins' );

class SearchTermHighlighter {
	
	var $option_name = 'hls_options';
	var $options = array();
	var $defaults = array(
		'content_id' => '#content',
		'colors' => array(
			'#D3E18A',
			'#FFCA61',
			'#9CD4FF',
			'#FA998C',
			'#FF6BA6',
		)
	 ); 
	
	var $plugin_file = '';
	var $plugin_basename = '';
	var $plugin_dir_url = '';
	
	function SearchTermHighlighter() {
		$this->plugin_file = __FILE__;
		$this->plugin_basename = plugin_basename($this->plugin_file);
		$this->plugin_dir_url = WP_PLUGIN_URL . "/" . dirname($this->plugin_basename);
		
		$this->load_options();
		
		add_action('init', array(&$this, 'hook_init'));
		add_action('wp_print_scripts', array(&$this, 'hook_wp_print_scripts'));
		add_action('wp_print_styles', array(&$this, 'hook_wp_print_styles'));
		add_action('wp_footer', array(&$this, 'hook_wp_footer'));
		
		add_action('activate_'.$this->plugin_basename, array(&$this,'activate'));
		add_action('plugin_action_links_'.$this->plugin_basename, array(&$this,'action_links'));
		add_action('admin_menu', array(&$this, 'hook_admin_menu'));
		
		//Add the colorpicker scripts to the settings page
		if ( strpos($_SERVER['REQUEST_URI'], 'page=highlight_search_terms')!==false ){
			wp_enqueue_script('jquery');
			//jQuery Colorpicker plugin
			wp_enqueue_script(
				'jquery-colorpicker-ws', 
				$this->plugin_dir_url.'/colorpicker/js/colorpicker.js', 
				array('jquery')
			 );
        }
	}
	
	function hook_init() {
		wp_enqueue_script('jquery');	
	}
	
	function remove_quotes($str){
		return str_replace(array('"', '\''), '', $str);
	}
	
	function hook_wp_print_scripts() {
		if (preg_match_all('/([^\s"\']+)|"([^"]*)"|\'([^\']*)\'/', get_search_query(), $terms)){
			$terms = $terms[0];
		} else {
			return;
		}
		
		if(count($terms) > 0){
			$filtered_terms = array();
			
			foreach($terms as $term){
				$term = attribute_escape(trim($this->remove_quotes($term)));
				if ( !empty($term) ){
					$filtered_terms[] = '"'.$term.'"';
				}
			}	
				
			if (count($filtered_terms) > 0) {
				echo '
				  <script type="text/javascript">
				    var hls_terms  = new Array('.implode(', ', $filtered_terms).');
				    var hls_color_count = '.count($this->options['colors']).';
				  </script>
				';
			}
		}
	}
	
	function hook_wp_print_styles() {
		echo '<style type="text/css" media="screen">';
		for ($i = 0; $i < count($this->options['colors']); $i++){
			echo ".hls-$i { background: ".$this->options['colors'][$i]."; }\n";
		}
		echo '</style>';
	}
	
	function hook_wp_footer(){
		?>
<script type="text/javascript">
  jQuery.fn.extend({
    highlight: function(search, insensitive, css_class){
      var regex = new RegExp("(<[^>]*>)|(\\b"+ search.replace(/([-.*+?^${}()|[\]\/\\])/g,"\\$1") +")", insensitive ? "ig" : "g");
      return this.html(this.html().replace(regex, function(a, b, c){
        return (a.charAt(0) == "<") ? a : "<strong class=\""+ css_class +"\">" + c + "</strong>";
      }));
    }
  });
  jQuery(document).ready(function($){
  	if (window.hls_terms != undefined){
  		for (i in hls_terms){
    		$("<?php echo js_escape($this->options['content_id']); ?>").highlight(hls_terms[i], 1, "hls-"+(i % hls_color_count));
    	}
    }
  });
</script>
		<?
	}
	
	function save_options(){
		if ($this->option_name)
			update_option($this->option_name, $this->options);
	}
	
	function load_options(){
		$this->options = get_option($this->option_name);
		if(!is_array($this->options)){
			$this->options = $this->defaults;
			return false;
		} else {
			$this->options = array_merge($this->defaults, $this->options);
			return true;
		}
	}
	
	function activate(){
		$this->save_options();
	}
	
	function action_links($links){
		$links[] = '<a href="options-general.php?page=highlight_search_terms">Settings</a>';
		return $links;
	}
	
	function hook_admin_menu(){
		$plugin_page = add_options_page(
			'Highlight Search Terms', 'Highlight Search Terms', 'manage_options', 
			'highlight_search_terms', array(&$this, 'page_options')
		 );
		add_action('admin_head-'.$plugin_page, array(&$this, 'hook_admin_head'));
	}
	
	function hook_admin_head(){
		$dir_url = WP_PLUGIN_URL . "/" . dirname($this->plugin_basename);
		echo '<link rel="stylesheet" media="screen" type="text/css" href="'.$dir_url.'/colorpicker/css/colorpicker.css" />';
	}
	
	function page_options(){
		
		if (!empty($_POST['action'])){
			if (check_admin_referer('update-hls-options')){
				
				$this->options['content_id'] = $_POST['content_id'];
				
				for ($i = 0; $i < count($this->options['colors']); $i++){
					if ( !empty($_POST['color_'.$i]) ){
						$this->options['colors'][$i] = $_POST['color_'.$i]; 
					}
				}
				
				$this->save_options();
				echo '<div id="message" class="updated fade"><p><strong>Settings saved</strong></p></div>';
			}
		}
		?>
<style type='text/css'>
.hls-button {
	display: inline;
	margin-left: 4px;
	vertical-align:middle;
	cursor: pointer;
}
</style>
<div class="wrap">
<?php if(function_exists('screen_icon')) screen_icon(); ?>
<h2>Highlight Search Terms</h2>

<form method="post">
<input type="hidden" name="action" value="update" />
<?php 
	wp_nonce_field('update-hls-options');
?>

<table class="form-table">

<tr valign="top">
<th scope="row">Content ID</th>
<td><input type="text" name="content_id" value="<?php echo $this->options['content_id'] ?>" />
<span class="setting-description">The HTML tag ID of the area where you want your search terms highlighted.</span></td>
</tr>

<?php
			foreach($this->options['colors'] as $num => $color){
				
				?>
<tr>
<th scope="row">Highlight Color #<?php echo $num+1; ?></th>
<td>
	<input type="text" id="hls-color-<?php echo $num; ?>" name="color_<?php echo $num; ?>" 
		class="hls-color-input" value="<?php echo $color; ?>"
		style="background-color: <?php echo $color; ?>; vertical-align:middle;" />
	<img class="hls-button" src="<? echo $this->plugin_dir_url,'/colorpicker/images/hls_button.png'; ?>"
		title="Open color picker" alt="Open color picker" />
</td>
</tr>				
				<?php
			}
?>

</table>

<p class="submit">
<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
</p>

</form>

</div>
<script type='text/javascript'>
jQuery(document).ready(function($){
	
	$('.hls-color-input').each(function(){
		$(this).keyup(function(){
			$(this).css('background-color', this.value);
		});
		
		var my_button = $(this).siblings('.hls-button')[0];
		$(my_button).attr('color_input', $(this).attr('id'));
		
		$(my_button).ColorPicker({
			onBeforeShow: function () {
				var field = $('#'+$(my_button).attr('color_input'))[0];
				$(my_button).ColorPickerSetColor(field.value);
			},
			onSubmit: function(hsb, hex, rgb) {
				var field = $('#'+$(my_button).attr('color_input'))[0];
				$(field).val('#'+hex.toUpperCase()).css('background-color', '#'+hex.toUpperCase());
			},
			onShow: function (colpkr) {
				$(colpkr).fadeIn(200);
				return false;
			},
			onHide: function (colpkr) {
				$(colpkr).fadeOut(200);
				return false;
			},
		});
	});
	
});
</script>
		<?
	}
}

$search_term_highlighter = new SearchTermHighlighter();
?>